#include <cstdio>
#include <iostream>
#include <vector>
#include <queue>
#include <set>
#define TRACE false
using namespace std;

int n, m, k;
vector<vector<int>> depth;
vector<vector<int>> gr;
vector<pair<int, int>> edges;
set<int> depths;

void read()
{
    cin>>n>>m>>k;
    gr.resize(n+1);
    depth.resize(n+1, vector<int>(n+1, 0));
    depths.insert(0);

    for(int i=0;i<m;i++)
    {
        int x, y, d;
        cin>>x>>y>>d;
        if(x==y)continue;
        depths.insert(d);

        if(depth[x][y]==0)
        {
            gr[x].push_back(y);
            gr[y].push_back(x);
            edges.emplace_back(min(x, y), max(x, y));
        }

        depth[x][y]+=d;
        depth[y][x]+=d;
    }
}

queue<int> q;
vector<bool> used;
int bfs(int from, int forbidden = -1)
{
    fill(used.begin(), used.end(), false);

    used[from]=true;
    q.push(from);

    #if TRACE
    printf("bfs from %d with f %d  : ", from, forbidden);
    #endif // TRACE

    int uCount = 0;
    while(!q.empty())
    {
        int x = q.front();
        #if TRACE
        printf("%d ", x);
        #endif // TRACE
        q.pop();
        uCount++;

        for(int y : gr[x])if(depth[x][y]!=0 && !used[y] && y!=forbidden)
        {
            used[y]=true;
            q.push(y);
        }
    }

    #if TRACE
    putchar('\n');
    #endif // TRACE

    return uCount;
}

bool check_blocked()
{
    return bfs(1)!=n;
}

bool check_bad(int x)
{
    bool isBad = (bfs(gr[x].front(), x)!=n-1);
    #if TRACE
    printf("%d is bad: %d\n", x, (int)isBad);
    #endif
    return isBad;
}

int count_bad(vector<int> &addTo)
{
    int badCount = 0;
    for(int i=1;i<=n;i++)
    {
        if(check_bad(i))
        {
            badCount++;
            addTo.push_back(i);
        }
    }
    return badCount;
}

void solve()
{
    #if TRACE
    for(int d : depths)printf("%d ", d);
    putchar('\n');
    #endif // TRACE

    used.resize(n+1);
    int prevDepth = 0;
    vector<int> bad;
    for(int d : depths)
    {
        bad.clear();
        int depthDelta = d-prevDepth;
        prevDepth=d;
        bool any = false;
        for(int x=1;x<=n;x++)for(int y : gr[x])
        {
            depth[x][y]=max(0, depth[x][y]-depthDelta);
            if(depth[x][y]!=0)any = true;
        }

        if(!any)break;

        #if TRACE
        for(int i=1;i<=n;i++)
        {
            for(int j=1;j<=n;j++)
            {
                printf("%d ", depth[i][j]);
            }
            putchar('\n');
        }
        putchar('\n');
        #endif // TRACE

        fill(used.begin(), used.end(), false);
        if(check_blocked())
        {
            puts("-1");
            return;
        }

        int badCount = count_bad(bad);
        if(badCount>=k)
        {
            printf("%d %d\n", d, badCount);
            for(int b : bad)printf("%d ", b);
            putchar('\n');
            return;
        }
    }

    puts("-1");

}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    read();
    solve();
    return EXIT_SUCCESS;
}

/**
8 13 3
1 2 19
1 3 13
2 3 5
4 2 9
2 5 7
3 5 17
6 3 8
3 8 12
4 7 11
5 7 5
7 2 10
7 6 13
8 5 16


3 3 1
1 2 42
2 3 42
1 3 666
**/

