#include <iostream>
#include <vector>
#include <deque>
using namespace std;

const char* alpha="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
int get_dig (char c)
{
    if(c>='A' && c<='Z') return c-'A'+10;
    return c-'0';
}
struct Number
{
    deque <int> dig;
    int base;
    bool cf;
    Number (const string& s, int base, bool cf=false): base(base), cf(cf)
    {
        for(const auto& i: s) dig.push_back(get_dig(i));
    }
    Number (const deque<int>& dig, int base, bool cf=false): dig(dig), base(base), cf(cf) {}
    Number (deque<int>&& dig, int base, bool cf=false): dig(move(dig)), base(move(base)), cf(cf) {}
    friend ostream& operator<< (ostream& os, const Number& n)
    {
        for(const auto& i: n.dig) os << alpha[i];
        return os;
    }
    Number operator+ (const Number& n) const
    {
        if(!dig.front()) return n;
        if(!n.dig.front()) return *this;
        int a[10000], carry=0;
        int *p=a+10000;
        auto it1=dig.rbegin();
        auto it2=n.dig.rbegin();
        while(it1!=dig.rend() && it2!=n.dig.rend())
        {
            int t=*it1+*it2+carry;
            if(t>=base) carry=1, t-=base;
            else carry=0;
            *--p=t;
            ++it1; ++it2;
        }
        while(it1!=dig.rend())
        {
            int t=*it1+carry;
            if(t>=base) carry=1, t-=base;
            else carry=0;
            *--p=t;
            ++it1;
        }
        while(it2!=n.dig.rend())
        {
            int t=*it2+carry;
            if(t>=base) carry=1, t-=base;
            else carry=0;
            *--p=t;
            ++it2;
        }
        bool CF;
        if(carry) *--p=1, CF=true;
        else CF=false;
        return Number(deque<int>(p,a+10000),base,CF);
    }
    Number operator* (int digit) const
    {
        int a[10000], carry=0;
        int *p=a+10000;
        for(auto it=dig.rbegin(); it!=dig.rend(); ++it)
        {
            int t=digit**it+carry;
            *--p=t%base;
            carry=t/base;
        }
        if(carry) *--p=carry;
        return Number(deque<int>(p,a+10000),base);
    }
    Number operator* (const Number& n) const
    {
        Number res("0",base);
        int c=0;
        for(auto it=dig.rbegin(); it!=dig.rend(); ++it)
        {
            Number t(n**it);
            t.dig.resize(t.dig.size()+c++,0);
            res=res+t;
        }
        return res;
    }
    void remLeading0()
    {
        while(dig.size()>1 && !dig.front()) dig.pop_front();
    }
    void remTrailing0()
    {
        while(dig.size()>1 && !dig.back()) dig.pop_back();
    }
    Number& operator++()
    {
        return *this=*this+Number("1",base);
    }
    Number& add0(int count)
    {
        while(count--) dig.push_back(0);
        return *this;
    }
    void remFirstDig()
    {
        dig.pop_front();
    }
    size_t size() const
    {
        return dig.size();
    }
};


int main()
{
    int radix;
    cin >> radix;
    string A, B;
    cin >> A >> B;
    size_t p1=A.find('.'), p2=B.find('.');
    Number Aint(A.substr(0,p1),radix), Bint(B.substr(0,p2),radix);
    Number Afrac(A.substr(p1+1),radix), Bfrac(B.substr(p2+1),radix);
    Aint.remLeading0(); Bint.remLeading0();
    if(Afrac.size()<Bfrac.size()) Afrac.add0(Bfrac.size()-Afrac.size());
    else Bfrac.add0(Afrac.size()-Bfrac.size());
    Afrac.remLeading0(); Bfrac.remLeading0();
    Number SumInt(Aint+Bint), SumFrac(Afrac+Bfrac);
    if(p1!=string::npos && p2!=string::npos)
    {
        if(SumFrac.cf)
        {
            SumFrac.remFirstDig();
            ++SumInt;
        }
        SumFrac.remTrailing0();
        cout << SumInt;
        if(SumFrac.size()!=1 || SumFrac.dig.back()) cout << '.' << SumFrac;
    }
    else cout << SumInt;
    cout << endl;
}

/**
5
1242.1(2)
002.322(22)
*/
