#include <bits/stdc++.h>
#define endl '\n'
using namespace std;

const int MAX_N = 5e4 + 5;
const int MAX_M = 2e5 + 5;

class Edge {
public:
	int x, y, s;

	bool operator< (const Edge &other) const {
		return s < other.s;
	}
};

class UnionFind {
private:
	int parent[MAX_N], compSize[MAX_N];

public:
	int cntComps;

	void Init(int cntNodes) {
		for(int i = 1; i <= cntNodes; i++) {
			parent[i] = i;
			compSize[i] = 1;
		}
		cntComps = cntNodes;
	}

	int Find(int x) {
		if(parent[x] == x) {
			return x;
		}
		else {
			parent[x] = Find(parent[x]);
			return parent[x];
		}
	}

	void Union(int x, int y) {
		cntComps--;
		if(compSize[x] <= compSize[y]) {
			parent[x] = y;
			compSize[y] += compSize[x];
		}
		else {
			parent[y] = x;
			compSize[x] += compSize[y];
		}
	}
};

UnionFind uf;
Edge edges[MAX_M];

class Graph {
public:
	class Node {
	public:
		bool isVisited, isDangerous;
		int id;
		vector< pair< int, int > > v;
	};

	int cntNodes;
	Node nodes[MAX_N];

	void Init(int cntNodes) {
		this->cntNodes = cntNodes;

		for(int i = 1; i <= cntNodes; i++) {
			nodes[i].id = i;
		}
	}

	void AddEdge(int x, int y, int z) {
		nodes[x].v.push_back({ y, z });
		nodes[y].v.push_back({ x, z });
	}

	void MarkAllNodesUnvisited() {
		for(int i = 1; i <= cntNodes; i++) {
			nodes[i].isVisited = false;
		}
	}
	
	void MarkAllNodesSafe() {
		for(int i = 1; i <= cntNodes; i++) {
			nodes[i].isDangerous = false;
		}
	}

	int Dfs(int node, int minEdge) {
		nodes[node].isVisited = true;
		
		int aux = 1;
		for(auto x : nodes[node].v) {
			if(x.second < minEdge || nodes[x.first].isVisited) {
				continue;
			}

			aux += Dfs(x.first, minEdge);
		}

		return aux;
	}

	int CountDangerous(int minEdge) {
		int cntDangerous = 0;
	
		MarkAllNodesSafe();
		for(int i = 1; i <= cntNodes; i++) {
			MarkAllNodesUnvisited();
			
			nodes[i].isVisited = true;

			int cntVisited = 0;
			for(auto x : nodes[i].v) {
				if(x.first != i && x.second >= minEdge) {
					cntVisited = Dfs(x.first, minEdge);
					break;
				}
			}
			
			if(cntVisited != cntNodes - 1) {
				nodes[i].isDangerous = true; 
				cntDangerous++;
			}
		}

		return cntDangerous;
	}
};

Graph g;

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

	int n, m, k;
	cin >> n >> m >> k;

	for(int i = 1; i <= m; i++) {
		cin >> edges[i].x >> edges[i].y >> edges[i].s;
	}

	sort(edges + 1, edges + 1 + m);

	uf.Init(n);

	int maxRemovedEdgeInd;
	for(int i = m; i >= 1; i--) {
		int parX = uf.Find(edges[i].x);
		int parY = uf.Find(edges[i].y);

		if(parX != parY) {
			uf.Union(parX, parY);

			if(uf.cntComps == 1) {
				int iCpy = i;
				while(edges[i].s == edges[iCpy].s) {
					i--;
				}
				
				maxRemovedEdgeInd = i;
				break;
			}
		}
	}
	
	g.Init(n);
	for(int i = m; i >= 1; i--) {
		g.AddEdge(edges[i].x, edges[i].y, edges[i].s);
	}

	int low = 0, high = maxRemovedEdgeInd, ans = -1;
	while(low <= high) {
		int mid = (low + high) / 2;

		if(g.CountDangerous(edges[mid].s + 1) >= k) {
			ans = edges[mid].s;
			high = mid - 1;
		}
		else {
			low = mid + 1;
		}
	}

	if(ans == -1) {
		cout << "-1 0" << endl;
	}
	else {
		g.CountDangerous(ans + 1);

		cout << ans << " ";

		vector< int > ansNodes;
		for(int i = 1; i <= n; i++) {
			if(g.nodes[i].isDangerous) {
				ansNodes.push_back(i);
			}
		}

		cout << ansNodes.size() << endl;
		for(int x : ansNodes) {
			cout << x << " ";
		}
		cout << endl;
	}

	return 0;
}
