#include <iostream>
#include <cstdint>
#include <vector>
#include <algorithm>
#include <map>

uint32_t base;

uint32_t ch_int(char c) {
	if ((c >= '0') && (c <= '9')) {
		return (c - '0');
	} else {
		return (c - 'A' + 10);
	}
}

char int_ch (uint32_t a) {
	if (a < 10) {
		return a + '0';
	} else {
		return a + 'A' - 10;
	}
}

struct Int {
	std::vector<uint32_t> dig;
	uint64_t hash() const {
		uint64_t a = 0;
		const uint32_t mod = 1e9 + 7;
		for (uint32_t i : dig) {
			a *= 31;
			a += i;
			a %= mod;
		}
		return a;
	}
	Int(uint32_t a = 0) {
		dig.push_back(a);
		while (dig.back() >= base) {
			uint32_t t = dig.back();
			dig.back() %= base;
			dig.push_back(t / base);
		}
	}
	uint32_t& operator[] (uint32_t i) {
		return dig[i];
	}
	const uint32_t& operator[] (uint32_t i) const {
		return dig[i];
	}
	uint32_t size() const {
		return dig.size();
	}

};

std::ostream& operator<< (std::ostream &s, const Int &n) {
	for (uint32_t i = n.size() - 1;i < n.size();i--) {
		s << int_ch(n[i]);
	}
	return s;
}

bool operator<= (Int &a, Int &b) {
	if (a.size() != b.size()) {
		return a.size() < b.size();
	}
	for (uint32_t i = a.size() - 1;i < a.size();i--) {
		if (a[i] != b[i]) {
			return a[i] < b[i];
		}
	}
	return true;
}

Int operator+ (const Int &a, const Int &b) {
	Int ans;
	ans.dig.resize(std::max(a.size(), b.size()) + 1);
	for (uint32_t i = 0;i < a.size();i++) {
		ans[i] += a[i];
	}
	for (uint32_t i = 0;i < b.size();i++) {
		ans[i] += b[i];
	}
	for (uint32_t i = 0;i < ans.size();i++) {
		if (ans[i] >= base) {
			ans[i + 1]++;
			ans[i] -= base;
		}
	}
	while ((ans.size() > 1) && (ans.dig.back() == 0)) {
		ans.dig.pop_back();
	}
	return ans;
}

Int operator- (const Int &a, const Int &b) {
	Int ans;
	std::cerr << a << " - " << b << std::endl;
	ans.dig.resize(a.size());
	for (uint32_t i = 0;i < a.size();i++) {
		ans[i] = a[i];
	}
	for (uint32_t i = 0;i < b.size();i++) {
		while ((ans[i] >= base)) {
			ans[i] += base;
			ans[i + 1]--;
		}

		if (ans[i] < b[i]) {
			ans[i] += base;
			ans[i + 1]--;
		}

		ans[i] -= b[i];
	}
	for (uint32_t i = 0;i < ans.size();i++) {
		while (ans[i] >= base) {
			ans[i + 1]--;
			ans[i] += base;
		}
	}
	while ((ans.size() > 1) && (ans.dig.back() == 0)) {
		ans.dig.pop_back();
	}
	return ans;
}
Int operator* (const Int &a, const Int &b) {
	Int ans;
	ans.dig.resize(a.size() + b.size());
	for (uint32_t i = 0;i < a.size();i++) {
		for (uint32_t j = 0;j < b.size();j++) {
			ans[i + j] += a[i] * b[j];
		}
	}
	for (uint32_t i = 0;i + 1 < ans.size();i++) {
		ans[i + 1] += ans[i] / base;
		ans[i] %= base;
	}
	while ((ans.size() > 1) && (ans.dig.back() == 0)) {
		ans.dig.pop_back();
	}
	std::cerr << a << " * " << b << " = " << ans << std::endl;
	return ans;
}

template <class T>
T fast_pow(const T& a, uint32_t b) {
	if (b == 1) {
		return a;
	} 
	T ans = fast_pow(a, b / 2);
	ans = ans * ans;
	if (b & 1) {
		ans = ans * a;
	}
	return ans;
}

struct Rat {
	Int p, q;

	friend Rat operator+ (const Rat &a, const Rat &b) {
		Rat ans;
		ans.q = a.q * b.q;
		ans.p = b.q * a.p + a.q * b.p;
		return ans;
	}
};

std::ostream& operator<< (std::ostream &s, const Rat &r) {
	return s << r.p << "\n---------\n" << r.q << "\n";
}


Rat read_rat() {
	std::string s;
	std::cin >> s;
	Int whole;
	whole.dig.clear();
	uint32_t i = 0;
	for (i = 0;i < s.size();i++) {
		if (s[i] == '.') {
			break;
		}
		whole.dig.push_back(ch_int(s[i]));
		
	}
	std::reverse(whole.dig.begin(), whole.dig.end());
	Rat ans;
	if (i == s.size()) {
		ans.p = whole;
		ans.q = Int(1);
		return ans;
	}

	Int dr;
	dr.dig.clear();
	for (i++;i < s.size();i++) {
		if (s[i] == '(') {
			break;
		}
		dr.dig.push_back(ch_int(s[i]));
	}
	std::reverse(dr.dig.begin(), dr.dig.end());
	if (dr.size() > 0) {
		ans.q = fast_pow(Int(10), dr.size());
		ans.p = ans.q * whole + dr;
	} else {
		ans.p = whole;
		ans.q = Int(1);
	}

	if (i == s.size()) {
		return ans;
	}

	Int per;
	per.dig.clear();
	for (i++;i < s.size();i++) {
		if (s[i] == ')') {
			break;
		}
		per.dig.push_back(ch_int(s[i]));
	}
	std::reverse(per.dig.begin(), per.dig.end());
	Rat t;
	Int m = (fast_pow(10, per.size()) - Int(1));
	std::cerr << "mult " <<  m << "\n";
	t.q = ans.q * (fast_pow(10, per.size()) - Int(1));
	t.p = per;

	return ans + t;
}

void pretty_print(Rat &r) {
	std::vector<uint32_t> ans;
	std::map<uint64_t, uint32_t> cHash;

	uint32_t wholeDig;
	if (r.p.size() >= r.q.size()) {
		wholeDig = r.p.size() - r.q.size() + 1;
	} else {
		wholeDig = 1;
	}

	if (wholeDig > 1) {
		r.q = r.q * fast_pow(10, wholeDig - 1);
	}
	uint32_t t = 0;
	while (true) {
		uint32_t c = 0;
		std::cerr << "div " << r.p << " " << r.q << "\n";
		t++;
		while (r.q <= r.p) {
			c++;
			r.p = r.p - r.q;
		}
		std::cerr << "hi " << c << "\n";
		ans.push_back(c);
		if ((r.p.size() == 1) && (r.p[0] == 0)) {
			break;
		}
		r.p = r.p * Int(10);
		if (t == 10) {
			break;
		}
	}
	bool found = false;
	for (uint32_t i = 0;i < ans.size();i++) {
		if ((!found) && (ans[i] == 0)) {
			if (wholeDig == i + 1) {
				std::cout << "0.";
				found = true;
			}
		} else {
			found = true;
			std::cout << int_ch(ans[i]);
			if (i + 1 == wholeDig) {
				std::cout << ".";
			}
		}
	}
	std::cout << "\n";
}

int main() {
	std::cin >> base;
	Rat r1 = read_rat();
	Rat r2 = read_rat();

	std::cerr << r1 << r2;

	Rat ans = r1 + r2;
	std::cerr << ans;

	//std::cerr << ans.p << "\n/\n" << ans.q << "\n";
	pretty_print(ans);
}

