#include <iostream>
#include <cstdint>
#include <vector>
#include <algorithm>

const uint32_t MAX_S = 1e9 + 2;

struct Vertex;

struct Edge {
	Vertex *to;
	uint32_t s;

	Edge(Vertex *pTo = nullptr, uint32_t pS = 0): to(pTo), s(pS) {}
};

struct Vertex {
	std::vector<Edge> edges;
	Vertex *par;
	uint32_t depth;
	bool vis;
	bool crit;

	uint32_t dfs(uint32_t snow);
};

std::vector<Vertex> graph;

uint32_t Vertex::dfs(uint32_t snow) {
	vis = true;
	uint32_t ans = depth;
	uint32_t a2 = depth;
	uint32_t activeE = depth > 0;
	uint32_t parCnt = 0;
	for (Edge &e : edges) {
		if (e.s <= snow) {
			continue;
		}

		if (e.to->vis) {
			if (e.to->depth + 1 == depth) {
				parCnt++;
			}
			if ((e.to->depth + 1 != depth) || (parCnt > 0)) {
				a2 = std::min(a2, e.to->depth);
			}
		} else {
			activeE++;
			//std::cerr << "es " << e.s << " " << snow << "\n";
			e.to->depth = depth + 1;
			//std::cerr << this - &graph[0] + 1<< " -> " << e.to - &graph[0] + 1 << "\n";
			ans = std::min(ans, e.to->dfs(snow));
		}
	}
	//std::cerr << "v " << this - &graph[0] + 1 << " d " << depth << " a  " << ans << " ae  " << activeE << "\n";

	if ((activeE > 1) && (ans == depth)) {
		crit = true;
	}
	ans = std::min(ans, a2);
	return ans;
}


uint32_t critTarg;
std::vector<uint32_t> snows;

void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	uint32_t vCnt, eCnt;
	std::cin >> vCnt >> eCnt >> critTarg;

	graph.resize(vCnt);
	for (uint32_t i = 0;i < eCnt;i++) {
		uint32_t a, b, s;
		std::cin >> a >> b >> s;
		a--;
		b--;
		snows.push_back(s);
		graph[a].edges.push_back(Edge(&graph[b], s));
		graph[b].edges.push_back(Edge(&graph[a], s));
	}
}

void reset_graph() {
	for (Vertex &v : graph) {
		v.depth = 0;
		v.vis = v.crit = false;
		v.par = nullptr;
	}
}

bool possib(uint32_t snow) {
	reset_graph();
	for (Vertex &v : graph) {
		if (!v.vis) {
			v.dfs(snow);
		}
	}

	uint32_t curCrit = 0;
	for (Vertex &v : graph) {
		curCrit += v.crit;
	}
	//std::cerr << snow << " " << curCrit << "\n";
	return curCrit >= critTarg;
}

int main() {
	input();
	snows.push_back(0);
	std::sort(snows.begin(), snows.end());
	for (uint32_t i : snows) {
		if (possib(i)) {
			std::vector<uint32_t> ans;
			for (uint32_t i = 0;i < graph.size();i++) {
				if (graph[i].crit) {
					ans.push_back(i + 1);
				}
			}
			std::cout << i << " " << ans.size() << "\n";
			for (uint32_t i : ans) {
				std::cout << i << " ";
			}
			std::cout << "\n";
			return 0;
		}
	}
	std::cout << "-1 0\n";

}
