#include <vector>
#include <utility>
#include <cstdint>
#include <iostream>
#include <cassert>
#include <queue>

#include "hunter.h"

const uint32_t INF = 1e9;
const uint32_t MAX_N = 101;

uint32_t dx[] = {0, 0, 1, (uint32_t)(-1)};
uint32_t dy[] = {1, (uint32_t)(-1), 0, 0};

inline uint32_t col(uint32_t x, uint32_t y) {
	return (x ^ y) & 1;
}

uint32_t h;
uint32_t w;

uint32_t cell_comp[MAX_N][MAX_N];

struct Vertex;

struct Edge {
	Vertex *to;
	uint32_t cap;
	uint32_t revInd;
	
	Edge(Vertex *pTo = nullptr, uint32_t pCap = 0): to(pTo), cap(pCap) {}
	void push_flow() {
		cap--;
		rev().cap++;
	}

	Edge& rev();
};

struct Vertex {
	std::vector<Edge> edges;
	uint32_t depth;
	uint32_t x, y;
	uint32_t vis;

	void add_edge(Vertex *to) {
		edges.push_back(Edge(to, 1));
		to->edges.push_back(Edge(this, 0));

		edges.back().revInd = to->edges.size() - 1;
		to->edges.back().revInd = edges.size() - 1;
	}

	Vertex(): vis(0) {}

	bool push_flow(uint32_t v, Vertex *sink) {
		if (this == sink) {
			return true;
		}

		vis = v;
		for (Edge &e : edges) {
			if ((e.to->vis < v) && (e.cap)) {
				if (e.to->push_flow(v, sink)) {
					e.push_flow();
					return true;
				}
			}
		}

		return false;
	}
};

Edge& Edge::rev() {
	return to->edges[revInd];
}

Vertex vert[MAX_N][MAX_N];

struct Comp {
	Vertex source;
	Vertex sink;
	uint32_t cnt[2];

	Comp() {
		cnt[0] = cnt[1] = 0;
	}
	void bfs() {
		std::queue<Vertex*> next;
		next.push(&source);
		while (!next.empty()) {
			//vCnt++;
			//Vertex.push_back(
		}
	}

	void gen_flow() {
		uint32_t cVis = 1;
		while (source.push_flow(cVis, &sink)) {
			cVis++;
		}
		//assert((cnt[0] == cVis - 1) || cnt[1] == cVis - 1);

	}
};

std::vector<Comp> comps;

void dfs(uint32_t x, uint32_t y) {
	for (uint32_t i = 0;i < 4;i++) {
		uint32_t nX = x + dx[i];
		uint32_t nY = y + dy[i];

		if ((nX >= w) || (nY >= h) || (cell_comp[nX][nY] > 0)) {
			continue;
		} else {
			cell_comp[nX][nY] = cell_comp[x][y];
			dfs(nX, nY);
		}
	}
}

void rem_comp(uint32_t c) {
	for (uint32_t x = 0;x < w;x++) {
		for (uint32_t y = 0;y < h;y++) {
			if (cell_comp[x][y] == c + 1) {
				cell_comp[x][y] = 0;
			}
		}
	}
}

std::pair<int32_t, int32_t> find_empty() {
	for (uint32_t x = 0;x < w;x++) {
		for (uint32_t y = 0;y < h;y++) {
			if (cell_comp[x][y] == 0) {
				return {x, y};
			}
		}
	}
	return {-1, -1};
}

void set_empty(uint32_t c) {
	for (uint32_t x = 0;x < w;x++) {
		for (uint32_t y = 0;y < h;y++) {
			if (cell_comp[x][y] == 0) {
				cell_comp[x][y] = c + 1;
			}
		}
	}
}

void calc_cell_comp(const std::vector<std::vector<char>> &table) {
	for (uint32_t y = 0;y < h;y++) {
		for (uint32_t x = 0;x < w;x++) {
			if (table[y][x] == '#') {
				cell_comp[x][y] = INF;
			} else {
				cell_comp[x][y] = 0;
			}
		}
	}
	uint32_t cCnt = 0;
	for (uint32_t x = 0;x < w;x++) {
		for (uint32_t y = 0;y < h;y++) {
			if (cell_comp[x][y] == 0) {
				cCnt++;
				cell_comp[x][y] = cCnt;
				dfs(x, y);
				comps.push_back(Comp());
			}
			if (cell_comp[x][y] != INF) {
				comps[cell_comp[x][y] - 1].cnt[col(x, y)]++;
			}

		}
	}
	//std::cerr << "comp cnt " << cCnt << std::endl;
}

uint32_t key_comp() {
	for (uint32_t i = comps.size() - 1;i < comps.size();i--) {
		//std::cerr << "cnt " << comps[i].cnt[0] << " " << comps[i].cnt[1] << std::endl;
		if (comps[i].cnt[0] != comps[i].cnt[1]) {
			return i;
		}
	}
	return INF;
}

std::pair<int32_t, int32_t> find_cell(uint32_t comp, uint32_t c) {
	std::vector<std::pair<int32_t, int32_t>> possib;
	for (uint32_t y = 0;y < h;y++) {
		for (uint32_t x = 0;x < w;x++) {
			if ((col(x, y) != c) || (cell_comp[x][y] - 1 != comp)) {
				continue;
			}
			possib.push_back({x, y});
			/*cell_comp[x][y] = INF;
			rem_comp(comp);
			auto p = find_empty();
			bool ok = true;

			if (p.first >= 0) {
				cell_comp[p.first][p.second] = comp + 1;
				//std::cerr << p.first << " " << p.second << std::endl;
				dfs(p.first, p.second);
				p = find_empty();
				if (p.first >= 0) {
					ok = false;
				}
			}
			set_empty(comp);
			cell_comp[x][y] = comp + 1;
			if (ok) {
				return {x, y};
			}
			*/
		}
	}
	return possib[std::min(0, (int32_t)(possib.size() / 2))];
	//return {-1, -1};

}

void build_graph() {
	for (uint32_t x = 0;x < w;x++) {
		for (uint32_t y = 0;y < h;y++) {
			if (cell_comp[x][y] == INF) {
				continue;
			}
			vert[x][y].x = x;
			vert[x][y].y = y;

			if (col(x, y) == 0) {
				comps[cell_comp[x][y] - 1].source.add_edge(&vert[x][y]);
				for (uint32_t i = 0;i < 4;i++) {
					uint32_t nX = x + dx[i];
					uint32_t nY = y + dy[i];

					if ((nX < w) && (nY < h) && (cell_comp[nX][nY] != INF)) {
						vert[x][y].add_edge(&vert[nX][nY]);
					}
				}
			} else {
				vert[x][y].add_edge(&comps[cell_comp[x][y] - 1].sink);
			}
		}
	}
}

std::pair<int32_t, int32_t> ans[MAX_N][MAX_N];

void calc_ans() {
	for (uint32_t x = 0;x < w;x++) {
		for (uint32_t y = 0;y < w;y++) {
			ans[x][y] = {-1, -1};
			if (cell_comp[x][y] == INF) {
				continue;
			}
			uint32_t c = col(x, y);
			Comp &comp = comps[cell_comp[x][y] - 1];
			for (Edge &e : vert[x][y].edges) {
				if ((e.cap == c) && (e.to != &comp.source) && (e.to != &comp.sink)) {
					ans[x][y] = {e.to->x, e.to->y};
				}
			}
		}
	}
}

void play(int32_t H, int32_t W, const std::vector<std::vector<char>> &table) {
	h = H;
	w = W;
	calc_cell_comp(table);

	uint32_t kC = key_comp();
	std::pair<int32_t, int32_t> sPos = {-1, -1};
	if (kC != INF) {
		Comp &c = comps[kC];
		uint32_t sCol = c.cnt[0] > c.cnt[1] ? 0 : 1;
		sPos = find_cell(kC, sCol);
		c.cnt[sCol]--;
		cell_comp[sPos.first][sPos.second] = INF;
	} 
	build_graph();
	for (Comp &c : comps) {
		c.gen_flow();
	}
	calc_ans();
	//std::cerr << "compCnt " << comps.size() << std::endl;
	//std::cerr << "key comp " << kC << std::endl;

	std::pair<int32_t, int32_t> curPos = makeMove(sPos.second, sPos.first);
	while (curPos.first >= 0) {
		auto a = ans[curPos.second][curPos.first];
		//if (a.first < 0) {
			//a.first = 8 / 0;
		//}
		curPos = makeMove(a.second, a.first);
	}
}
